/* code07d0.c - Preferences panel
   Copyright  1999 Mitch Blevins <mblevin@debian.org>
   GNU GPL License
   Based on LinkHistoryHack by Ben Darnel <bgdarnel@unity.ncsu.edu>
   which was...
   Based on SampleHack by Roger Chaplin <foursquaredev@att.net>
*/


#include <Pilot.h>
#include "findIgnoreHack.h"
#include "findIgnoreHackRsc.h"
#include "callback.h"

static void PrefsInit(FormPtr frm);
static void PrefsOK();
static void PrefsListDrawFunc(UInt index, RectanglePtr bounds, CharPtr* data);
static void DoListSelect(short selection);
static void ScrollList(enum directions direction);

// This is the code that will be called whenever the user taps myhack's "+"
// button in the HackMaster list. HackMaster is not smart about the content of
// this file; it will simply jump to the first executable statement in this
// file. If you need helper functions, prototype them above, and define them
// AFTER the trap patch.
Boolean PrefsHandler(EventPtr event)
{
   FormPtr         frm;
   Boolean handled=false;

   switch (event->eType) {
   case frmOpenEvent:
      frm = FrmGetActiveForm();
      PrefsInit(frm);
      FrmDrawForm(frm);
      handled=1;
      break;
      
   case ctlSelectEvent:
      switch (event->data.ctlEnter.controlID) {
      case PrefsOKButton:
	 PrefsOK();
	 /* FALL THRU */

      case PrefsCancelButton:
	 FrmGotoForm(9000);  // return to HackMaster control panel
	 handled=true;
	 break;
      }
      break;

   case lstSelectEvent:
      DoListSelect(event->data.lstSelect.selection);
      break;

   case keyDownEvent:
      switch (event->data.keyDown.chr) {
          case pageUpChr:
              ScrollList(up);
              break;
          case pageDownChr:
              ScrollList(down);
              break;
      }
      break;
      
   default:
      break;
   }
   
   return handled;
}

static void PrefsInit(FormPtr frm) {
    Handle happlist;
    SysDBListItemType *papplist;
    Word appcount;
    ListPtr list;
    UInt i, j;
    MyPrefsType prefs;
    Word prefssize=sizeof(prefs);

    /* Get the excluded app list */
    MemSet(&prefs, sizeof(MyPrefsType), 0);
    if ((PrefGetAppPreferences('FIHK', 0, &prefs, &prefssize, true)
	                    ==noPreferenceFound)||(prefssize<sizeof(prefs))) {
        MemSet(&prefs, sizeof(MyPrefsType), 0);
    }

    /* Get every stinking little application */
    if (SysCreateDataBaseList('appl', 0, &appcount, &happlist, true)) {
   
        /* Publish the database list handle */
        FtrSet(MYCRID, 0x115c /* 4444 - why not? */, (DWord)happlist);
        FtrSet(MYCRID, 0x115d /* 4445 - why not? */, (DWord)appcount);
        /* Cheat and use version to signify an excluded application */
        papplist = MemHandleLock((VoidHand)happlist);
        for (i=0; i<appcount; i++) {
            papplist[i].version = 0;
            for (j=0; j<64; j++) {
                if (!prefs.creatorID[j]) break;
                if (papplist[i].creator == prefs.creatorID[j] ) {
                    papplist[i].version = 1;
                }
            }
        }
        MemHandleUnlock((VoidHand)happlist);

        list = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, AppList));
        LstSetListChoices(list, NULL, appcount);
        LstSetDrawFunction(list, PrefsListDrawFunc);
        LstSetSelection(list, -1);
    }
    else {
        FtrUnregister(MYCRID, 0x115c);
        FtrUnregister(MYCRID, 0x115d);
    }


}

static void PrefsOK() {
    SysDBListItemType *papplist;
    Handle happlist;
    MyPrefsType prefs;
    DWord appcount;
    UInt i, j;
    Word prefssize=sizeof(prefs);

    MemSet(&prefs, sizeof(MyPrefsType), 0);

    /* Get the applist */
    if (FtrGet(MYCRID, 0x115c, (DWord*)&happlist)) return;
    if (FtrGet(MYCRID, 0x115d, &appcount)) return;
    papplist = MemHandleLock((VoidHand)happlist);

    j = 0;
    for (i=0; i<appcount; i++) {
        if (papplist[i].version) {
            prefs.creatorID[j] = papplist[i].creator;
            if (j >=63) break;
            else j++;
        }
    }

    PrefSetAppPreferences('FIHK', 0, 0, &prefs, prefssize, true);
    /* Cleanup */
    MemHandleUnlock((VoidHand)happlist);
}

static void PrefsListDrawFunc(UInt index, RectanglePtr bounds, CharPtr* data) {
    SysDBListItemType *papplist;
    Handle happlist;
    VoidHand checkH;
    CALLBACK_PROLOGUE

    /* Get the applist */
    if (FtrGet(MYCRID, 0x115c, (DWord*)&happlist)) {
        WinDrawChars("Error", 5, bounds->topLeft.x, bounds->topLeft.y);
        return;
    }
    papplist = MemHandleLock((VoidHand)happlist);

    /* Draw the application name */
    if (papplist[index].version) {
        checkH = DmGetResource('Tbmp', CheckBitmap);
        WinDrawBitmap((BitmapPtr)MemHandleLock(checkH), bounds->topLeft.x,
                        bounds->topLeft.y);
        MemHandleUnlock(checkH);
        DmReleaseResource(checkH);
    }
    WinDrawChars(papplist[index].name, StrLen(papplist[index].name),
                            bounds->topLeft.x+15, bounds->topLeft.y);

    /* Cleanup */
    MemHandleUnlock((VoidHand)happlist);
    CALLBACK_EPILOGUE
}

static void DoListSelect(short selection) {
    FormPtr frm;
    ListPtr list;
    SysDBListItemType *papplist;
    Handle happlist;

    if (selection < 0) return;
    if (FtrGet(MYCRID, 0x115c, (DWord*)&happlist)) return;
    papplist = MemHandleLock((VoidHand)happlist);
    if (papplist[selection].version) papplist[selection].version = 0;
    else papplist[selection].version = 1;
    MemHandleUnlock((VoidHand)happlist);
    frm = FrmGetActiveForm();
    list = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, AppList));
    LstEraseList(list);
    LstSetSelection(list, -1);
    LstDrawList(list);
}

static void ScrollList(enum directions direction) {
    FormPtr frm;
    ListPtr list;

    frm = FrmGetActiveForm();
    list = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, AppList));

    LstScrollList(list, direction, 7);
}
